/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

(function() {
    /**
     * self - The timerWorker Object.
     * time - The total time.
     * currentTime - The remaining time.
     * isRunning - When this timer is activated, change to true
     * intervalHandler - The object handles setInterval event.
     */
    var self = this,
        time,
        currentTime,
        isRunning,
        intervalHandler;

    /**
     * Post message every second.
     * @private
     */
    function timerOn() {
        currentTime--;
        self.postMessage(currentTime);

        if (currentTime === 0) {
            self.postMessage("PAUSE");
            clearInterval(intervalHandler);
            isRunning = false;
            currentTime = time;
        }
    }


    /**
     * The onMessage eventHandler of timerWorker
     * @private
     */
    self.onmessage = function(e) {
        if (e.data.message === "INIT") {
            // When receive "INIT" message, initialize timerWorker.
            time = e.data.time;
            currentTime = e.data.time;
            isRunning = false;
        } else if (e.data.message === "START") {
            // When receive "START" message, toggle state of isRunning and set or clear interval event.
            if (isRunning) {
                self.postMessage("PAUSE");
                clearInterval(intervalHandler);
                isRunning = false;
            } else {
                self.postMessage("START");
                intervalHandler = setInterval(timerOn, 1000);
                isRunning = true;
            }
        } else if (e.data.message === "RESET") {
            // When receive "RESET" message, change isRunning to false and clear interval event.
            clearInterval(intervalHandler);
            isRunning = false;
            currentTime = time;
            self.postMessage("RESET");
        }
    };
}());